@@ -1,18 +1,19 @@
-# Copyright (C) 2007-2009 Osmo Salomaa
+# -*- coding: utf-8 -*-
+
+# Copyright (C) 2007 Osmo Salomaa
 #
-# This file is part of Gaupol.
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 #
-# Gaupol is free software: you can redistribute it and/or modify it under the
-# terms of the GNU General Public License as published by the Free Software
-# Foundation, either version 3 of the License, or (at your option) any later
-# version.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
 #
-# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
-# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# Gaupol. If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """Data store and basic position manipulation of a single subtitle."""
 
@@ -22,30 +23,32 @@ import copy
 __all__ = ("Subtitle",)
 
 
-class Subtitle(object):
+class Subtitle:
 
-    """Data store and basic position manipulation of a single subtitle.
+    """
+    Data store and basic position manipulation of a single subtitle.
 
-    :ivar start: Start position in native units (read/write)
-    :ivar start_time: Start time as string (read)
-    :ivar start_frame: Start frame as integer (read)
-    :ivar start_seconds: Start seconds as float (read)
-    :ivar end: End position in native units (read/write)
-    :ivar end_time: End time as string (read)
-    :ivar end_frame: End frame as integer (read)
-    :ivar end_seconds: End seconds as float (read)
-    :ivar duration: Duration in native units (read/write)
-    :ivar duration_time: Duration in time as string (read)
-    :ivar duration_frame: Duration in frames as integer (read)
-    :ivar duration_seconds: Duration in seconds as float (read)
-    :ivar main_text: Main text (read/write)
-    :ivar tran_text: Translation text (read/write)
-    :ivar calc: :class:`aeidon.Calculator` instance used (read/write)
-    :ivar framerate: :attr:`aeidon.framerates` item (read/write)
-    :ivar mode: :attr:`aeidon.modes` item (read/write)
+    :ivar start: Start position in native units
+    :ivar start_time: Start time as string
+    :ivar start_frame: Start frame as integer
+    :ivar start_seconds: Start seconds as float
+    :ivar end: End position in native units
+    :ivar end_time: End time as string
+    :ivar end_frame: End frame as integer
+    :ivar end_seconds: End seconds as float
+    :ivar duration: Duration in native units
+    :ivar duration_time: Duration in time as string
+    :ivar duration_frame: Duration in frames as integer
+    :ivar duration_seconds: Duration in seconds as float
+    :ivar main_text: Main text
+    :ivar tran_text: Translation text
+    :ivar calc: :class:`aeidon.Calculator` instance used
+    :ivar framerate: :attr:`aeidon.framerates` item
+    :ivar mode: :attr:`aeidon.modes` item
 
-    Positions can be set as strings for times, integers for frames or floats
-    for seconds. Positions are saved internally in only one mode.
+    Positions can be set as times, frames or seconds.
+    Use :func:`aeidon.as_time`, :func:`aeidon.as_frame` or
+    :func:`aeidon.as_seconds` if necessary to ensure correct type.
 
     Additional format-specific attributes are kept under separate containers,
     e.g. ``ssa`` for Sub Station Alpha formats, accessed as ``subtitle.ssa.*``.
@@ -53,18 +56,28 @@ class Subtitle(object):
     instantiation and excessive memory use when handling simpler formats.
     """
 
-    def __cmp__(self, other):
-        """Compare start positions in this subtitle's mode.
+    def __init__(self, mode=None, framerate=None):
+        """Initialize a :class:`Subtitle` instance."""
+        self._start = None
+        self._end = None
+        self._main_text = ""
+        self._tran_text = ""
+        self._mode = mode or aeidon.modes.TIME
+        self._framerate = framerate or aeidon.framerates.FPS_23_976
+        self.calc = aeidon.Calculator(self._framerate)
+        self.start = "00:00:00.000"
+        self.end = "00:00:00.000"
 
-        Return 1 if this subtitle appears later, 0 if they appear at the same
-        time and -1 if `other` appears later.
-        """
-        if self._mode == aeidon.modes.TIME:
-            times = (self._start, other.start_time)
-            return self.calc.compare_times(*times)
-        if self._mode == aeidon.modes.FRAME:
-            return cmp(self._start, other.start_frame)
-        raise ValueError("Invalid mode: %s" % repr(self._mode))
+    def __eq__(self, other):
+        """Compare subtitle equality by value."""
+        if not isinstance(other, Subtitle):
+            raise NotImplementedError
+        return (self.start == other.start and
+                self.end == other.end and
+                self.main_text == other.main_text and
+                self.tran_text == other.tran_text and
+                self.framerate == other.framerate and
+                self.mode == other.mode)
 
     def __getattr__(self, name):
         """Return lazily instantiated format-specific attribute container."""
@@ -73,47 +86,75 @@ class Subtitle(object):
             container = aeidon.containers.new(name)
             object.__setattr__(self, name, container)
             return container
-        raise AttributeError("Invalid container name: %s" % repr(name))
+        raise AttributeError("Invalid container name: {}"
+                             .format(repr(name)))
 
-    def __init__(self, mode=None, framerate=None):
-        """Initialize a :class:`Subtitle` object."""
-        self._start = "00:00:00.000"
-        self._end = "00:00:00.000"
-        self._main_text = ""
-        self._tran_text = ""
-        self._mode = mode or aeidon.modes.TIME
-        self._framerate = framerate or aeidon.framerates.FPS_23_976
-        self.calc = aeidon.Calculator(self._framerate)
+    def __ge__(self, other):
+        """Compare start positions."""
+        if self._mode == aeidon.modes.TIME:
+            return self.start_seconds >= other.start_seconds
+        if self._mode == aeidon.modes.FRAME:
+            return self.start_frame >= other.start_frame
+        raise ValueError("Invalid mode: {}"
+                         .format(repr(self._mode)))
 
-    def _convert_position(self, value):
-        """Return `value` of position in correct mode."""
-        if isinstance(value, basestring):
-            if self._mode == aeidon.modes.TIME:
-                return value
-            if self._mode == aeidon.modes.FRAME:
-                return self.calc.time_to_frame(value)
-        if isinstance(value, int):
-            if self._mode == aeidon.modes.TIME:
-                return self.calc.frame_to_time(value)
-            if self._mode == aeidon.modes.FRAME:
-                return value
-        if isinstance(value, float):
-            if self._mode == aeidon.modes.TIME:
-                return self.calc.seconds_to_time(value)
-            if self._mode == aeidon.modes.FRAME:
-                return self.calc.seconds_to_frame(value)
-        raise ValueError("Invalid type for value: %s" % repr(type(value)))
+    def __gt__(self, other):
+        """Compare start positions."""
+        if self._mode == aeidon.modes.TIME:
+            return self.start_seconds > other.start_seconds
+        if self._mode == aeidon.modes.FRAME:
+            return self.start_frame > other.start_frame
+        raise ValueError("Invalid mode: {}"
+                         .format(repr(self._mode)))
+
+    def __le__(self, other):
+        """Compare start positions."""
+        if self._mode == aeidon.modes.TIME:
+            return self.start_seconds <= other.start_seconds
+        if self._mode == aeidon.modes.FRAME:
+            return self.start_frame <= other.start_frame
+        raise ValueError("Invalid mode: {}"
+                         .format(repr(self._mode)))
+
+    def __lt__(self, other):
+        """Compare start positions."""
+        if self._mode == aeidon.modes.TIME:
+            return self.start_seconds < other.start_seconds
+        if self._mode == aeidon.modes.FRAME:
+            return self.start_frame < other.start_frame
+        raise ValueError("Invalid mode: {}"
+                         .format(repr(self._mode)))
 
     def convert_framerate(self, framerate):
         """Set framerate and convert positions to it."""
         coefficient = framerate.value / self._framerate.value
-        self.framerate = framerate
         if self._mode == aeidon.modes.TIME:
-            self.start = self.start_seconds / coefficient
-            self.end = self.end_seconds / coefficient
+            self.start_seconds = self.start_seconds / coefficient
+            self.end_seconds = self.end_seconds / coefficient
         if self._mode == aeidon.modes.FRAME:
-            self.start = int(round(coefficient * self.start_frame, 0))
-            self.end = int(round(coefficient * self.end_frame, 0))
+            self.start_frame = round(coefficient * self.start_frame)
+            self.end_frame = round(coefficient * self.end_frame)
+        self.framerate = framerate
+
+    def _convert_position(self, value):
+        """Return `value` of position in correct mode."""
+        if aeidon.is_time(value):
+            if self._mode == aeidon.modes.TIME:
+                return value
+            if self._mode == aeidon.modes.FRAME:
+                return self.calc.time_to_frame(value)
+        if aeidon.is_frame(value):
+            if self._mode == aeidon.modes.TIME:
+                return self.calc.frame_to_time(value)
+            if self._mode == aeidon.modes.FRAME:
+                return value
+        if aeidon.is_seconds(value):
+            if self._mode == aeidon.modes.TIME:
+                return self.calc.seconds_to_time(value)
+            if self._mode == aeidon.modes.FRAME:
+                return self.calc.seconds_to_frame(value)
+        raise ValueError("Invalid type for value: {}"
+                         .format(repr(type(value))))
 
     def copy(self):
         """Return a new subtitle instance with the same values."""
@@ -124,7 +165,7 @@ class Subtitle(object):
         subtitle._tran_text = self._tran_text
         # Copy all containers that have been instantiated.
         containers = (x.container for x in aeidon.formats)
-        for name in (set(dir(self)) & set(containers)):
+        for name in set(dir(self)) & set(containers):
             container = copy.deepcopy(getattr(self, name))
             setattr(subtitle, name, container)
         return subtitle
@@ -136,35 +177,44 @@ class Subtitle(object):
             return self.duration_time
         if self._mode == aeidon.modes.FRAME:
             return self.duration_frame
-        raise ValueError("Invalid mode: %s" % repr(self._mode))
+        raise ValueError("Invalid mode: {}"
+                         .format(repr(self._mode)))
 
     @duration.setter
     def duration(self, value):
         """Set duration from `value`."""
         value = self._convert_position(value)
-        if self._mode == aeidon.modes.TIME:
-            times = (self.start_time, value)
-            self._end = self.calc.add_times(*times)
-        if self._mode == aeidon.modes.FRAME:
-            self._end = self.start_frame + value
+        self._end = self.calc.add(self._start, value)
 
     @property
     def duration_frame(self):
         """Return duration as frames."""
-        bounds = (self.start_frame, self.end_frame)
-        return self.calc.get_frame_duration(*bounds)
+        return self.end_frame - self.start_frame
+
+    @duration_frame.setter
+    def duration_frame(self, value):
+        """Set duration from `value`."""
+        self.duration = aeidon.as_frame(value)
 
     @property
     def duration_seconds(self):
         """Return duration as seconds."""
-        time = self.duration_time
-        return self.calc.time_to_seconds(time)
+        return self.end_seconds - self.start_seconds
+
+    @duration_seconds.setter
+    def duration_seconds(self, value):
+        """Set duration from `value`."""
+        self.duration = aeidon.as_seconds(value)
 
     @property
     def duration_time(self):
         """Return duration as time."""
-        bounds = (self.start_time, self.end_time)
-        return self.calc.get_time_duration(*bounds)
+        return self.calc.seconds_to_time(self.duration_seconds)
+
+    @duration_time.setter
+    def duration_time(self, value):
+        """Set duration from `value`."""
+        self.duration = aeidon.as_time(value)
 
     @property
     def end(self):
@@ -183,13 +233,23 @@ class Subtitle(object):
             return self.calc.time_to_frame(self._end)
         if self._mode == aeidon.modes.FRAME:
             return self._end
-        raise ValueError("Invalid mode: %s" % repr(self._mode))
+        raise ValueError("Invalid mode: {}"
+                         .format(repr(self._mode)))
+
+    @end_frame.setter
+    def end_frame(self, value):
+        """Set end position from `value`."""
+        self.end = aeidon.as_frame(value)
 
     @property
     def end_seconds(self):
         """Return end position as seconds."""
-        time = self.end_time
-        return self.calc.time_to_seconds(time)
+        return self.calc.time_to_seconds(self.end_time)
+
+    @end_seconds.setter
+    def end_seconds(self, value):
+        """Set end position from `value`."""
+        self.end = aeidon.as_seconds(value)
 
     @property
     def end_time(self):
@@ -198,7 +258,13 @@ class Subtitle(object):
             return self._end
         if self._mode == aeidon.modes.FRAME:
             return self.calc.frame_to_time(self._end)
-        raise ValueError("Invalid mode: %s" % repr(self._mode))
+        raise ValueError("Invalid mode: {}"
+                         .format(repr(self._mode)))
+
+    @end_time.setter
+    def end_time(self, value):
+        """Set end position from `value`."""
+        self.end = aeidon.as_time(value)
 
     @property
     def framerate(self):
@@ -217,7 +283,8 @@ class Subtitle(object):
             return self.duration_time
         if mode == aeidon.modes.FRAME:
             return self.duration_frame
-        raise ValueError("Invalid mode: %s" % repr(mode))
+        raise ValueError("Invalid mode: {}"
+                         .format(repr(mode)))
 
     def get_end(self, mode):
         """Return end position in `mode`."""
@@ -225,7 +292,8 @@ class Subtitle(object):
             return self.end_time
         if mode == aeidon.modes.FRAME:
             return self.end_frame
-        raise ValueError("Invalid mode: %s" % repr(mode))
+        raise ValueError("Invalid mode: {}"
+                         .format(repr(mode)))
 
     def get_start(self, mode):
         """Return start position in `mode`."""
@@ -233,7 +301,8 @@ class Subtitle(object):
             return self.start_time
         if mode == aeidon.modes.FRAME:
             return self.start_frame
-        raise ValueError("Invalid mode: %s" % repr(mode))
+        raise ValueError("Invalid mode: {}"
+                         .format(repr(mode)))
 
     def get_text(self, doc):
         """Return text corresponding to `doc`."""
@@ -241,7 +310,8 @@ class Subtitle(object):
             return self._main_text
         if doc == aeidon.documents.TRAN:
             return self._tran_text
-        raise ValueError("Invalid document: %s" % repr(doc))
+        raise ValueError("Invalid document: {}"
+                         .format(repr(doc)))
 
     def has_container(self, name):
         """Return ``True`` if container has been instantiated."""
@@ -255,7 +325,7 @@ class Subtitle(object):
     @main_text.setter
     def main_text(self, value):
         """Set main text from `value`."""
-        self._main_text = unicode(value)
+        self._main_text = value
 
     @property
     def mode(self):
@@ -276,11 +346,11 @@ class Subtitle(object):
     def scale_positions(self, value):
         """Multiply start and end positions by `value`."""
         if self._mode == aeidon.modes.TIME:
-            self.start = self.start_seconds * value
-            self.end = self.end_seconds * value
+            self.start_seconds = self.start_seconds * value
+            self.end_seconds = self.end_seconds * value
         if self._mode == aeidon.modes.FRAME:
-            self.start = int(round(self._start * value, 0))
-            self.end = int(round(self._end * value, 0))
+            self.start_frame = round(self._start * value)
+            self.end_frame = round(self._end * value)
 
     def set_text(self, doc, value):
         """Set text corresponding to `doc` to `value`."""
@@ -291,21 +361,8 @@ class Subtitle(object):
 
     def shift_positions(self, value):
         """Add `value` to start and end positions."""
-        if isinstance(value, basestring):
-            start = self.calc.add_times(self.start_time, value)
-            end = self.calc.add_times(self.end_time, value)
-            self.start = start
-            self.end = end
-        elif isinstance(value, int):
-            start = self.start_frame + value
-            end = self.end_frame + value
-            self.start = start
-            self.end = end
-        elif isinstance(value, float):
-            start = self.start_seconds + value
-            end = self.end_seconds + value
-            self.start = start
-            self.end = end
+        self._start = self.calc.add(self._start, value)
+        self._end = self.calc.add(self._end, value)
 
     @property
     def start(self):
@@ -324,13 +381,23 @@ class Subtitle(object):
             return self.calc.time_to_frame(self._start)
         if self._mode == aeidon.modes.FRAME:
             return self._start
-        raise ValueError("Invalid mode: %s" % repr(self._mode))
+        raise ValueError("Invalid mode: {}"
+                         .format(repr(self._mode)))
+
+    @start_frame.setter
+    def start_frame(self, value):
+        """Set start position from `value`."""
+        self.start = aeidon.as_frame(value)
 
     @property
     def start_seconds(self):
         """Return start position as seconds."""
-        time = self.start_time
-        return self.calc.time_to_seconds(time)
+        return self.calc.time_to_seconds(self.start_time)
+
+    @start_seconds.setter
+    def start_seconds(self, value):
+        """Set start position from `value`."""
+        self.start = aeidon.as_seconds(value)
 
     @property
     def start_time(self):
@@ -339,7 +406,13 @@ class Subtitle(object):
             return self._start
         if self._mode == aeidon.modes.FRAME:
             return self.calc.frame_to_time(self._start)
-        raise ValueError("Invalid mode: %s" % repr(self._mode))
+        raise ValueError("Invalid mode: {}"
+                         .format(repr(self._mode)))
+
+    @start_time.setter
+    def start_time(self, value):
+        """Set start position from `value`."""
+        self.start = aeidon.as_time(value)
 
     @property
     def tran_text(self):
@@ -349,4 +422,4 @@ class Subtitle(object):
     @tran_text.setter
     def tran_text(self, value):
         """Set translation text from `value`."""
-        self._tran_text = unicode(value)
+        self._tran_text = value
