@@ -63,6 +63,7 @@ class MenuAgent(aeidon.Delegate):
         self.connect("page-saved", self._update_projects_menu)
         self.connect("page-switched", self._update_projects_menu)
         self.connect("pages-reordered", self._update_projects_menu)
+        self.connect("init-done", self._update_recent_menus)
         self.connect("page-added", self._update_recent_menus)
         self.connect("page-saved", self._update_recent_menus)
 
@@ -104,8 +105,13 @@ class MenuAgent(aeidon.Delegate):
         for item in menu.get_children():
             menu.remove(item)
         self._redo_menu_items = []
-        page = self.get_current_page()
-        for i, action in enumerate(page.project.redoables):
+        redoables = []
+        with aeidon.util.silent(AttributeError):
+            # XXX: Gtk.Actionable.set_action_name doesn't affect the dropdown
+            # arrow making it clickable even when there's nothing to redo.
+            page = self.get_current_page()
+            redoables = page.project.redoables
+        for i, action in enumerate(redoables):
             item = Gtk.MenuItem(label=action.description)
             item.gaupol_index = i
             item.connect("activate", self._on_redo_menu_item_activate)
@@ -161,8 +167,13 @@ class MenuAgent(aeidon.Delegate):
         for item in menu.get_children():
             menu.remove(item)
         self._undo_menu_items = []
-        page = self.get_current_page()
-        for i, action in enumerate(page.project.undoables):
+        undoables = []
+        with aeidon.util.silent(AttributeError):
+            # XXX: Gtk.Actionable.set_action_name doesn't affect the dropdown
+            # arrow making it clickable even when there's nothing to undo.
+            page = self.get_current_page()
+            undoables = page.project.undoables
+        for i, action in enumerate(undoables):
             item = Gtk.MenuItem(label=action.description)
             item.gaupol_index = i
             item.connect("activate", self._on_undo_menu_item_activate)
@@ -269,17 +280,25 @@ class MenuAgent(aeidon.Delegate):
             action = "win.open-recent-main-file-{:d}".format(i)
             menu.append(label, action)
             action = action.replace("win.", "")
-            if not self.get_action(action):
-                ao = gaupol.Action(action)
-                callback = self._on_open_recent_main_file_activate
-                ao.connect("activate", callback)
-                self.window.add_action(ao)
-            self.get_action(action).gaupol_path = path
+            if self.get_action(action):
+                # If action i exists, update the file path.
+                self.get_action(action).gaupol_path = path
+            else:
+                # Otherwise, create the action and add to self.window.
+                # XXX: For some reason, this can sometimes fail.
+                with aeidon.util.silent(Exception):
+                    ao = gaupol.Action(action)
+                    ao.gaupol_path = path
+                    callback = self._on_open_recent_main_file_activate
+                    ao.connect("activate", callback)
+                    self.window.add_action(ao)
 
     def _update_recent_menus(self, *args):
         """Update the file menu lists of recent files."""
         self._update_recent_main_menu()
         self._update_recent_translation_menu()
+        # Update enabled states of added actions.
+        self.update_gui()
 
     def _update_recent_translation_menu(self, *args):
         """Update the file menu list of recent translation files."""
@@ -297,7 +316,7 @@ class MenuAgent(aeidon.Delegate):
             menu.append(label, action)
             action = action.replace("win.", "")
             if not self.get_action(action):
-                ao = gaupol.Action(action)
+                ao = gaupol.OpenRecentTranslationFileAction(action)
                 callback = self._on_open_recent_translation_file_activate
                 ao.connect("activate", callback)
                 self.window.add_action(ao)
